@@ -26,6 +26,7 @@ import sys
 import time
 import zlib
 import base64
+import random as rnd_module
 import inspect
 import warnings
 import traceback
@@ -39,12 +40,12 @@ from coverage.files import canonical_filename
 from coverage.collector import Collector
 
 import hypothesis.strategies as st
-from hypothesis import __version__
 from hypothesis.errors import Flaky, Timeout, NoSuchExample, \
     Unsatisfiable, DidNotReproduce, InvalidArgument, DeadlineExceeded, \
     MultipleFailures, FailedHealthCheck, HypothesisWarning, \
     UnsatisfiedAssumption, HypothesisDeprecationWarning
 from hypothesis.control import BuildContext
+from hypothesis.version import __version__
 from hypothesis._settings import Phase, Verbosity, HealthCheck, \
     PrintSettings
 from hypothesis._settings import settings as Settings
@@ -74,14 +75,17 @@ try:
 except ImportError:  # pragma: no cover
     from coverage.collector import FileDisposition
 
+if False:
+    from typing import Any, Dict, Callable, Optional, Union  # noqa
+    from hypothesis.utils.conventions import InferType  # noqa
+
 
 running_under_pytest = False
 global_force_seed = None
 
 
 def new_random():
-    import random
-    return random.Random(random.getrandbits(128))
+    return rnd_module.Random(rnd_module.getrandbits(128))
 
 
 @attr.s()
@@ -304,8 +308,7 @@ def get_random_for_wrapped_test(test, wrapped_test):
     elif global_force_seed is not None:
         return Random(global_force_seed)
     else:
-        import random
-        seed = random.getrandbits(128)
+        seed = rnd_module.getrandbits(128)
         wrapped_test._hypothesis_internal_use_generated_seed = seed
         return Random(seed)
 
@@ -424,7 +427,7 @@ class Arc(object):
         self.target = target
 
 
-ARC_CACHE = {}
+ARC_CACHE = {}  # type: Dict[str, Dict[Any, Dict[Any, Arc]]]
 
 
 def arc(filename, source, target):
@@ -450,7 +453,6 @@ class StateForActualGivenExecution(object):
         self.test_runner = test_runner
         self.search_strategy = search_strategy
         self.settings = settings
-        self.at_least_one_success = False
         self.last_exception = None
         self.falsifying_examples = ()
         self.__was_flaky = False
@@ -552,7 +554,6 @@ class StateForActualGivenExecution(object):
                 data.can_reproduce_example_from_repr = True
             with self.settings:
                 with BuildContext(data, is_final=is_final):
-                    import random as rnd_module
                     rnd_module.seed(0)
                     args, kwargs = data.draw(self.search_strategy)
                     if expected_failure is not None:
@@ -678,12 +679,11 @@ class StateForActualGivenExecution(object):
                             arc(filename, source, target)
                             for source, target in covdata.arcs(filename)
                         )
-            if result is not None and self.settings.perform_health_check:
+            if result is not None:
                 fail_health_check(self.settings, (
                     'Tests run under @given should return None, but '
                     '%s returned %r instead.'
                 ) % (self.test.__name__, result), HealthCheck.return_value)
-            self.at_least_one_success = True
             return False
         except UnsatisfiedAssumption:
             data.mark_invalid()
@@ -776,31 +776,20 @@ class StateForActualGivenExecution(object):
                 ) % (
                     self.settings.timeout, runner.valid_examples),
                     self.settings)
-            if runner.valid_examples < min(
-                self.settings.min_satisfying_examples,
-                self.settings.max_examples,
-            ) and not (
-                runner.exit_reason == ExitReason.finished and
-                self.at_least_one_success
-            ):
+            if runner.valid_examples == 0:
                 if timed_out:
                     raise Timeout((
                         'Ran out of time before finding a satisfying '
-                        'example for '
-                        '%s. Only found %d examples in ' +
-                        '%.2fs.'
+                        'example for %s. Only found %d examples in %.2fs.'
                     ) % (
                         get_pretty_function_description(self.test),
                         runner.valid_examples, run_time
                     ))
                 else:
-                    raise Unsatisfiable((
-                        'Unable to satisfy assumptions of hypothesis '
-                        '%s. Only %d examples considered '
-                        'satisfied assumptions'
-                    ) % (
-                        get_pretty_function_description(self.test),
-                        runner.valid_examples,))
+                    raise Unsatisfiable(
+                        'Unable to satisfy assumptions of hypothesis %s.' %
+                        (get_pretty_function_description(self.test),)
+                    )
 
         if not self.falsifying_examples:
             return
@@ -902,7 +891,11 @@ def fake_subTest(self, msg=None, **__):
     yield
 
 
-def given(*given_arguments, **given_kwargs):
+def given(
+    *given_arguments,  # type: Union[SearchStrategy, InferType]
+    **given_kwargs  # type: Union[SearchStrategy, InferType]
+):
+    # type: (...) -> Callable[[Callable[..., None]], Callable[..., None]]
     """A decorator for turning a test function that accepts arguments into a
     randomized test.
 
@@ -1088,15 +1081,21 @@ def given(*given_arguments, **given_kwargs):
     return run_test_with_generator
 
 
-def find(specifier, condition, settings=None, random=None, database_key=None):
+def find(
+    specifier,  # type: SearchStrategy
+    condition,  # type: Callable[[Any], bool]
+    settings=None,  # type: Settings
+    random=None,   # type: Any
+    database_key=None,  # type: bytes
+):
+    # type: (...) -> Any
     """Returns the minimal example from the given strategy ``specifier`` that
     matches the predicate function ``condition``."""
     settings = settings or Settings(
         max_examples=2000,
-        min_satisfying_examples=0,
         max_shrinks=2000,
     )
-    settings = Settings(settings, perform_health_check=False)
+    settings = Settings(settings, suppress_health_check=HealthCheck.all())
 
     if database_key is None and settings.database is not None:
         database_key = function_digest(condition)
@@ -1128,7 +1127,7 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
         if success:
             successful_examples[0] += 1
 
-        if settings.verbosity == Verbosity.verbose:
+        if settings.verbosity >= Verbosity.verbose:
             if not successful_examples[0]:
                 report(
                     u'Tried non-satisfying example %s' % (nicerepr(result),))
@@ -1159,12 +1158,11 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
             list(runner.interesting_examples.values())[0].buffer)
         with BuildContext(data):
             return data.draw(search)
-    if (
-        runner.valid_examples <= settings.min_satisfying_examples and
+    if runner.valid_examples == 0 and (
         runner.exit_reason != ExitReason.finished
     ):
         if settings.timeout > 0 and run_time > settings.timeout:
-            raise Timeout((
+            raise Timeout((  # pragma: no cover
                 'Ran out of time before finding enough valid examples for '
                 '%s. Only %d valid examples found in %.2f seconds.'
             ) % (
@@ -1172,11 +1170,9 @@ def find(specifier, condition, settings=None, random=None, database_key=None):
                 runner.valid_examples, run_time))
 
         else:
-            raise Unsatisfiable((
-                'Unable to satisfy assumptions of '
-                '%s. Only %d examples considered satisfied assumptions'
-            ) % (
-                get_pretty_function_description(condition),
-                runner.valid_examples,))
+            raise Unsatisfiable(
+                'Unable to satisfy assumptions of %s.' %
+                (get_pretty_function_description(condition),)
+            )
 
     raise NoSuchExample(get_pretty_function_description(condition))
